@@ -111,6 +111,10 @@
 #include "asimagexml.h"
 #include "transform.h"
 
+#ifdef jmpbuf
+#undef jmpbuf
+#endif
+
 
 /***********************************************************************************/
 /* High level interface : 														   */
@@ -134,7 +138,7 @@ as_image_loader_func as_image_file_loaders[ASIT_Unknown] =
 	tiff2ASImage,
 	xml2ASImage ,
 	NULL,
-	NULL,
+	tga2ASImage,
 	NULL
 };
  
@@ -325,6 +329,29 @@ get_asimage( ASImageManager* imageman, const char *file, ASFlagType what, unsign
 	return im;
 }
 
+void
+destroy_asimage_list( ASImageListEntry **plist )
+{
+	if( plist )
+	{		   
+		ASImageListEntry *curr = *plist ;
+		while( curr )
+		{
+			ASImageListEntry *to_delete = curr ; 
+			curr = curr->next ;
+			if( to_delete->preview ) 
+				safe_asimage_destroy( to_delete->preview );
+			if( to_delete->name )
+				free( to_delete->name );
+			if( to_delete->fullfilename )
+				free( to_delete->fullfilename );
+			free( to_delete );
+		}
+		*plist = NULL ;
+	}
+}
+
+
 ASImageListEntry *
 get_asimage_list( ASVisual *asv, const char *dir,
 	              ASFlagType preview_type, double gamma,
@@ -455,6 +482,10 @@ locate_image_file( const char *file, char **paths )
 				register int i = 0;
 				do
 				{
+					if( i > 0 ) 
+					{	
+						show_progress( "looking for image \"%s\" in path [%s]", file, paths[i] );
+					}		
 					realfilename = find_file( file, paths[i], R_OK );
 				}while( realfilename == NULL && paths[i++] != NULL );
 			}
@@ -770,7 +801,7 @@ png2ASImage( const char * path, ASImageImportParams *params )
 					color_type = PNG_COLOR_TYPE_GRAY_ALPHA ;
   */
 				if (png_get_sRGB (png_ptr, info_ptr, &intent))
-                    png_set_sRGB (png_ptr, info_ptr, DEFAULT_PNG_IMAGE_GAMMA);
+                    png_set_gamma (png_ptr, params->gamma, DEFAULT_PNG_IMAGE_GAMMA);
 				else if (png_get_gAMA (png_ptr, info_ptr, &image_gamma))
 					png_set_gamma (png_ptr, params->gamma, image_gamma);
 				else
@@ -827,7 +858,6 @@ png2ASImage( const char * path, ASImageImportParams *params )
 						register unsigned int i;
 						for ( i = 0 ; i < buf.width ; ++i)
 						{
-							/*fprintf( stderr, "%2.2X          ", buf.alpha[i] );*/
 							if( buf.alpha[i] != 0x00FF )
 							{	
 								if( buf.alpha[i] == 0 )
@@ -990,13 +1020,13 @@ jpeg2ASImage( const char * path, ASImageImportParams *params )
 		(void)jpeg_read_scanlines (&cinfo, buffer, 1);
 		if( cinfo.output_components==1 ) 
 		{	
-			apply_gamma( buffer[0], params->gamma_table, im->width );
-			im->channels[IC_RED][y] = store_data( NULL, buffer[0], im->width, ASStorage_RLEDiffCompress, 0);
+			apply_gamma( (CARD8*)buffer[0], params->gamma_table, im->width );
+			im->channels[IC_RED][y] = store_data( NULL, (CARD8*)buffer[0], im->width, ASStorage_RLEDiffCompress, 0);
 			im->channels[IC_GREEN][y] = dup_data( NULL, im->channels[IC_RED][y] );
 			im->channels[IC_BLUE][y]  = dup_data( NULL, im->channels[IC_RED][y] );
 		}else
 		{		   
-			raw2scanline( buffer[0], &buf, params->gamma_table, im->width, (cinfo.output_components==1), False);
+			raw2scanline( (CARD8*)buffer[0], &buf, params->gamma_table, im->width, (cinfo.output_components==1), False);
 			im->channels[IC_RED][y] = store_data( NULL, (CARD8*)buf.red, buf.width*4, ASStorage_32BitRLE, 0);
 			im->channels[IC_GREEN][y] = store_data( NULL, (CARD8*)buf.green, buf.width*4, ASStorage_32BitRLE, 0);
 			im->channels[IC_BLUE][y] = store_data( NULL, (CARD8*)buf.blue, buf.width*4, ASStorage_32BitRLE, 0);
@@ -1637,7 +1667,7 @@ tiff2ASImage( const char * path, ASImageImportParams *params )
 		return NULL;   
 	}		
 
-	if( rows_per_strip < 0 || rows_per_strip > height ) 
+	if( rows_per_strip == 0 || rows_per_strip > height ) 
 		rows_per_strip = height ;
 	if( depth <= 0 ) 
 		depth = 4 ;
@@ -1667,7 +1697,7 @@ tiff2ASImage( const char * path, ASImageImportParams *params )
 			
 			while( first_row < height ) 
 			{	
-				if (TIFFReadRGBAStrip(tif, first_row, data))
+				if (TIFFReadRGBAStrip(tif, first_row, (void*)data))
 				{
 					register CARD32 *row = data ;
 					int y = first_row + rows_per_strip ;
@@ -1767,3 +1797,222 @@ xml2ASImage( const char *path, ASImageImportParams *params )
 	SHOW_TIME("image loading",started);
 	return im ;
 }
+/*************************************************************************/
+/* Targa Image format - some stuff borrowed from the GIMP.
+ *************************************************************************/
+typedef struct ASTGAHeader
+{
+	CARD8 IDLength ;
+	CARD8 ColorMapType;
+#define TGA_NoImageData			0
+#define TGA_ColormappedImage	1
+#define TGA_TrueColorImage		2
+#define TGA_BWImage				3
+#define TGA_RLEColormappedImage		9
+#define TGA_RLETrueColorImage		10
+#define TGA_RLEBWImage				11
+	CARD8 ImageType;
+	struct 
+	{
+		CARD16 FirstEntryIndex ;
+		CARD16 ColorMapLength ;  /* number of entries */ 
+		CARD8  ColorMapEntrySize ;  /* number of bits per entry */ 
+	}ColormapSpec;
+	struct
+	{		
+		CARD16 XOrigin;
+		CARD16 YOrigin;
+		CARD16 Width;
+		CARD16 Height;
+		CARD8  Depth;
+#define TGA_LeftToRight		(0x01<<4)
+#define TGA_TopToBottom		(0x01<<5)
+		CARD8  Descriptor;
+	}ImageSpec;
+
+}ASTGAHeader;
+
+typedef struct ASTGAColorMap
+{
+	int bytes_per_entry;
+	int bytes_total ; 
+	CARD8 *data ; 
+}ASTGAColorMap;
+
+typedef struct ASTGAImageData
+{
+	int bytes_per_pixel;
+	int image_size;
+	int bytes_total ; 
+	CARD8 *data ; 
+}ASTGAImageData;
+
+static Bool load_tga_colormapped(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+static Bool load_tga_truecolor(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+	CARD32 *a = buf->alpha ;
+	CARD32 *r = buf->red ;
+	CARD32 *g = buf->green ;
+	CARD32 *b = buf->blue ;
+	int bpp = (tga->ImageSpec.Depth+7)/8;
+	int bpl = buf->width*bpp;
+	if( fread( read_buf, 1, bpl, infile ) != bpl ) 		   
+		return False;
+	if( bpp == 3 ) 
+	{	
+		int i;
+		if( gamma_table )
+			for( i = 0 ; i < buf->width ; ++i ) 
+			{
+				b[i] = gamma_table[*(read_buf++)];	
+				g[i] = gamma_table[*(read_buf++)];	  
+				r[i] = gamma_table[*(read_buf++)];	  
+			}	 
+		else
+			for( i = 0 ; i < buf->width ; ++i ) 
+			{
+				b[i] = *(read_buf++);	
+				g[i] = *(read_buf++);	  
+				r[i] = *(read_buf++);	  
+			}	 
+		set_flags( buf->flags, SCL_DO_RED|SCL_DO_GREEN|SCL_DO_BLUE );
+	}else if( bpp == 4 )
+	{
+		int i;
+		for( i = 0 ; i < buf->width ; ++i ) 
+		{
+			b[i] = *(read_buf++);	
+			g[i] = *(read_buf++);	  
+			r[i] = *(read_buf++);	  
+			a[i] = *(read_buf++);	  
+		}	 
+		set_flags( buf->flags, SCL_DO_RED|SCL_DO_GREEN|SCL_DO_BLUE|SCL_DO_ALPHA );
+	}	 
+
+	return True;
+}
+
+static Bool load_tga_bw(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+static Bool load_tga_rle_colormapped(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+static Bool load_tga_rle_truecolor(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+static Bool load_tga_rle_bw(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+
+
+ASImage *
+tga2ASImage( const char * path, ASImageImportParams *params )
+{
+	ASImage *im = NULL ;
+	/* More stuff */
+	FILE         *infile;					   /* source file */
+	ASTGAHeader   tga;
+	ASTGAColorMap *cmap = NULL ;
+	int width = 1, height = 1;
+	START_TIME(started);
+
+
+	if ((infile = open_image_file(path)) == NULL)
+		return NULL;
+	if( fread( &tga, 1, 3, infile ) == 3 ) 
+	if( fread( &tga.ColormapSpec, 1, 5, infile ) == 5 ) 
+	if( fread( &tga.ImageSpec, 1, 10, infile ) == 10 ) 
+	{
+		Bool success = True ;
+		Bool (*load_row_func)(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table );
+
+		if( tga.IDLength > 0 ) 
+			success = (fseek( infile, tga.IDLength, SEEK_CUR )==0);
+		if( success && tga.ColorMapType != 0 ) 
+		{
+			cmap = safecalloc( 1, sizeof(ASTGAColorMap));
+			cmap->bytes_per_entry = (tga.ColormapSpec.ColorMapEntrySize+7)/8;
+			cmap->bytes_total = cmap->bytes_per_entry*tga.ColormapSpec.ColorMapLength; 
+			cmap->data = safemalloc( cmap->bytes_total);
+			success = ( fread( cmap->data, 1, cmap->bytes_total, infile ) == cmap->bytes_total );
+		}else if( tga.ImageSpec.Depth != 24 && tga.ImageSpec.Depth != 32 )
+			success = False ;
+	 
+		if( success ) 
+		{
+			success = False;
+			if( tga.ImageType != TGA_NoImageData )
+			{	
+				width = tga.ImageSpec.Width ; 
+				height = tga.ImageSpec.Height ; 
+				if( width < MAX_IMPORT_IMAGE_SIZE && height < MAX_IMPORT_IMAGE_SIZE )
+					success = True;
+			}
+		}
+		switch( tga.ImageType ) 
+		{
+			case TGA_ColormappedImage	:load_row_func = load_tga_colormapped ; break ;
+			case TGA_TrueColorImage		:load_row_func = load_tga_truecolor ; break ;
+			case TGA_BWImage			:load_row_func = load_tga_bw ; break ;
+			case TGA_RLEColormappedImage:load_row_func = load_tga_rle_colormapped ; break ;
+			case TGA_RLETrueColorImage	:load_row_func = load_tga_rle_truecolor ; break ;
+			case TGA_RLEBWImage			:load_row_func = load_tga_rle_bw ; break ;
+			default:
+				load_row_func = NULL ;
+		}	 
+		
+		if( success && load_row_func != NULL ) 
+		{	
+			ASImageOutput  *imout ;
+			im = create_asimage( width, height, params->compression );
+			if((imout = start_image_output( NULL, im, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT)) == NULL )
+			{
+        		destroy_asimage( &im );
+				success = False;
+			}else
+			{	
+				ASScanline    buf;
+				int y ;
+				CARD8 *read_buf = safemalloc( width*4*2 ); 
+				prepare_scanline( im->width, 0, &buf, True );
+				if( !get_flags( tga.ImageSpec.Descriptor, TGA_TopToBottom ) )			
+					toggle_image_output_direction( imout );
+				for( y = 0 ; y < height ; ++y ) 
+				{	
+					if( !load_row_func( infile, &tga, cmap, &buf, read_buf, params->gamma_table ) )
+						break;
+					imout->output_image_scanline( imout, &buf, 1);
+				}
+				stop_image_output( &imout );
+				free_scanline( &buf, True );
+				free( read_buf );
+			}   
+		}	  
+	}	 
+	if( im == NULL )
+		show_error( "invalid or unsupported TGA format in image file \"%s\"", path );
+
+	fclose( infile );
+	SHOW_TIME("image loading",started);
+	return im ;
+}
+
+
