@@ -153,19 +153,20 @@ ValidatePixmap (Pixmap p, int bSetHandler, int bTransparent, unsigned int *pWidt
     /* we need to check if pixmap is still valid */
 	Window root;
     int junk;
+	unsigned int ujunk ;
 	if (bSetHandler)
 		oldXErrorHandler = XSetErrorHandler (pixmap_error_handler);
 
     if (bTransparent)
 	    p = GetRootPixmap (None);
 	if (!pWidth)
-  		pWidth = &junk;
+  		pWidth = &ujunk;
     if (!pHeight)
-	    pHeight = &junk;
+	    pHeight = &ujunk;
 
     if (p != None)
 	{
-  		if (!XGetGeometry (dpy, p, &root, &junk, &junk, pWidth, pHeight, &junk, &junk))
+  		if (!XGetGeometry (dpy, p, &root, &junk, &junk, pWidth, pHeight, &ujunk, &ujunk))
 			p = None;
     }
 	if(bSetHandler)
@@ -187,7 +188,7 @@ GetRootDimensions (int *width, int *height)
     if( dpy == NULL )
         return 0;
 	if (!XGetGeometry (dpy, RootWindow(dpy,DefaultScreen(dpy)), &root,
-					     &w_x, &w_y, width, height, &junk, &junk))
+					     &w_x, &w_y, (unsigned int *)width, (unsigned int *)height, &junk, &junk))
     {
     	*width = 0;
     	*height = 0;
@@ -202,15 +203,10 @@ int
 GetWinPosition (Window w, int *x, int *y)
 {
 #ifndef X_DISPLAY_MISSING
-	Window root, parent, *children;
-	unsigned int nchildren;
+	int bRes = 1;
 	static int rootWidth = 0, rootHeight = 0;
-	XWindowAttributes attr;
 	int my_x, my_y;
-
-	XGetWindowAttributes (dpy, w, &attr);
-	if (attr.map_state != IsViewable)
-  		return 0;
+	Window wdumm;
 
 	if (!x)
   		x = &my_x;
@@ -224,36 +220,21 @@ GetWinPosition (Window w, int *x, int *y)
   		if (!GetRootDimensions (&rootWidth, &rootHeight))
     		return 0;
 
-	while (XQueryTree (dpy, w, &root, &parent, &children, &nchildren))
-    {
-    	int w_x, w_y;
-    	unsigned int w_w, w_h, border_w, w_depth;
-    	if (children)
-			XFree (children);
-    	if (!XGetGeometry (dpy, w, &root,
-			 &w_x, &w_y, &w_w, &w_h, &border_w, &w_depth))
-			break;
-    	(*x) += w_x + border_w;
-    	(*y) += w_y + border_w;
-
-    	if (parent == root)
-		{			/* taking in to consideration virtual desktopping */
-			int bRes = 1;
-			if (*x < 0 || *x >= rootWidth || *y < 0 || *y >= rootHeight)
-	  			bRes = 0;
-			/* don't want to return position outside the screen even if we fail */
-			while (*x < 0)
-	  			*x += rootWidth;
-			while (*y < 0)
-	  			*y += rootHeight;
-			if (*x > rootWidth)
-	  			*x %= rootWidth;
-			if (*y > rootHeight)
-	  			*y %= rootHeight;
-			return bRes;
-		}
-    	w = parent;
-    }
+	
+	XTranslateCoordinates (dpy, w, RootWindow(dpy,DefaultScreen(dpy)), 0, 0, x, y, &wdumm);
+	/* taking in to consideration virtual desktopping */
+	if (*x < 0 || *x >= rootWidth || *y < 0 || *y >= rootHeight)
+		bRes = 0;
+	/* don't want to return position outside the screen even if we fail */
+	while(*x < 0)
+		*x += rootWidth;
+	while (*y < 0)
+		*y += rootHeight;
+	if (*x > rootWidth)
+		*x %= rootWidth;
+	if (*y > rootHeight)
+		*y %= rootHeight;
+	return bRes;
 #endif
 	*x = 0;
 	*y = 0;
@@ -265,10 +246,10 @@ shading2tint32(ShadingInfo * shading)
 {
 	if( shading && !NO_NEED_TO_SHADE(*shading))
 	{
-		CARD16 r16 = ((shading->tintColor.red*shading->shading / 100)>>1)&0x7FFF ;
-		CARD16 g16 = ((shading->tintColor.green*shading->shading / 100)>>1)&0x7FFF ;
-		CARD16 b16 = ((shading->tintColor.blue*shading->shading / 100)>>1)&0x7FFF ;
-		CARD16 a16 = ((0x0000FFFF*shading->shading / 100)>>1)&0x7FFF ;
+		CARD16 r16 = ((shading->tintColor.red*shading->shading / 100)>>9)&0x00FF ;
+		CARD16 g16 = ((shading->tintColor.green*shading->shading / 100)>>9)&0x00FF ;
+		CARD16 b16 = ((shading->tintColor.blue*shading->shading / 100)>>9)&0x00FF ;
+		CARD16 a16 = ((0x0000007F*shading->shading / 100))&0x00FF ;
 		return MAKE_ARGB32(a16,r16,g16,b16);
 	}
 	return TINT_LEAVE_SAME ;
@@ -286,9 +267,9 @@ scale_pixmap (ASVisual *asv, Pixmap src, int src_w, int src_h, int width, int he
 		src_im = pixmap2ximage(asv, src, 0, 0, src_w, src_h, AllPlanes, 0 );
 		if( src_im ) 
 		{
-			if( src_w != width && src_h != height ) 
+			if( src_w != width || src_h != height ) 
 			{
-				ASImage *tmp = scale_asimage( asv, src_im, width, height,
+				ASImage *tmp = scale_asimage( asv, src_im, width, height, 
 				                              (tint != TINT_LEAVE_SAME)?ASA_ASImage:ASA_XImage, 
 											  0, ASIMAGE_QUALITY_DEFAULT );
 				destroy_asimage( &src_im );
@@ -319,12 +300,7 @@ ScalePixmap (Pixmap src, int src_w, int src_h, int width, int height, GC gc, Sha
 {
 	Pixmap p = None ;
 #ifndef X_DISPLAY_MISSING
-	ASVisual asv ;
-	int screen = DefaultScreen(dpy);
-	
-	create_asvisual( dpy, screen, DefaultDepth(dpy, screen), &asv );
-	p = scale_pixmap( &asv, src, src_w, src_h, width, height, gc, shading2tint32(shading) );
-	destroy_asvisual( &asv, True );
+	p = scale_pixmap( get_default_asvisual(), src, src_w, src_h, width, height, gc, shading2tint32(shading) );
 #endif
 	return p;
 }
@@ -369,7 +345,6 @@ CopyAndShadeArea ( Drawable src, Pixmap trg,
 				   GC gc, ShadingInfo * shading)
 {
 #ifndef X_DISPLAY_MISSING
-	int screen = DefaultScreen(dpy);
 	ARGB32 tint = shading2tint32( shading );
 
     if (x < 0 || y < 0)
@@ -380,10 +355,7 @@ CopyAndShadeArea ( Drawable src, Pixmap trg,
 		XCopyArea (dpy, src, trg, gc, x, y, w, h, trg_x, trg_y);
 	}else
 	{
-		ASVisual asv ;
-		create_asvisual( dpy, screen, DefaultDepth(dpy, screen), &asv );
-		copyshade_drawable_area( &asv, src, trg, x, y, w, h, trg_x, trg_y, gc, tint );
-		destroy_asvisual( &asv, True );
+		copyshade_drawable_area( get_default_asvisual(), src, trg, x, y, w, h, trg_x, trg_y, gc, tint );
 	}
 #endif
 }
@@ -422,18 +394,8 @@ void
 ShadeTiledPixmap (Pixmap src, Pixmap trg, int src_w, int src_h, int x, int y, int w, int h, GC gc, ShadingInfo * shading)
 {
 #ifndef X_DISPLAY_MISSING
-	ASVisual *asv = NULL;
 	ARGB32 tint = shading2tint32( shading );
-  
-	if( tint != TINT_LEAVE_SAME )
-	{
-		int screen = DefaultScreen(dpy);
-	    asv = create_asvisual( dpy, screen, DefaultDepth(dpy, screen), NULL );
-	}
-
-	tile_pixmap (asv, src, trg, src_w, src_h, x, y, w, h, gc, tint);
-	if( asv )
-		destroy_asvisual( asv, False );
+	tile_pixmap (get_default_asvisual(), src, trg, src_w, src_h, x, y, w, h, gc, tint);
 #endif
 }
 
@@ -456,21 +418,13 @@ ShadePixmap (Pixmap src, int x, int y, int width, int height, GC gc, ShadingInfo
 {
     Pixmap trg = None;
 #ifndef X_DISPLAY_MISSING
-	ASVisual *asv = NULL;
 	ARGB32 tint = shading2tint32( shading );
   
-	if( tint != TINT_LEAVE_SAME )
-	{
-		int screen = DefaultScreen(dpy);
-	    asv = create_asvisual( dpy, screen, DefaultDepth(dpy, screen), NULL );
-	}
-	trg = CREATE_TRG_PIXMAP (asv, width, height);
+	trg = CREATE_TRG_PIXMAP (get_default_asvisual(), width, height);
 	if (trg != None)
     {
-    	copyshade_drawable_area (asv, src, trg, x, y, width, height, 0, 0, gc, tint);
+    	copyshade_drawable_area (get_default_asvisual(), src, trg, x, y, width, height, 0, 0, gc, tint);
     }
-	if( asv )
-		destroy_asvisual( asv, False );
 #endif
 	return trg;
 }
@@ -516,17 +470,9 @@ CenterPixmap (Pixmap src, int src_w, int src_h, int width, int height, GC gc, Sh
 {
 	Pixmap trg = None;
 #ifndef X_DISPLAY_MISSING
-	ASVisual *asv = NULL;
 	ARGB32 tint = shading2tint32( shading );
   
-	if( tint != TINT_LEAVE_SAME )
-	{
-		int screen = DefaultScreen(dpy);
-	    asv = create_asvisual( dpy, screen, DefaultDepth(dpy, screen), NULL );
-	}
-	trg = center_pixmap( asv, src, src_w, src_h, width, height, gc, tint );
-	if( asv )
-		destroy_asvisual( asv, False );
+	trg = center_pixmap( get_default_asvisual(), src, src_w, src_h, width, height, gc, tint );
 #endif		
 	return trg ;	
 }
@@ -558,17 +504,9 @@ GrowPixmap (Pixmap src, int src_w, int src_h, int width, int height, GC gc, Shad
 {
 	Pixmap trg = None;
 #ifndef X_DISPLAY_MISSING
-	ASVisual *asv = NULL;
 	ARGB32 tint = shading2tint32( shading );
   
-	if( tint != TINT_LEAVE_SAME )
-	{
-		int screen = DefaultScreen(dpy);
-	    asv = create_asvisual( dpy, screen, DefaultDepth(dpy, screen), NULL );
-	}
-	trg = grow_pixmap( asv, src, src_w, src_h, width, height, gc, tint );
-	if( asv )
-		destroy_asvisual( asv, False );
+	trg = grow_pixmap( get_default_asvisual(), src, src_w, src_h, width, height, gc, tint );
 #endif		
 	return trg ;	
 }
@@ -729,13 +667,8 @@ CutPixmap ( Pixmap src, Pixmap trg,
 {
 	Pixmap res = None;
 #ifndef X_DISPLAY_MISSING
-	ASVisual *asv = NULL;
 	ARGB32 tint = shading2tint32( shading );
-	int screen = DefaultScreen(dpy);
-    asv = create_asvisual( dpy, screen, DefaultDepth(dpy, screen), NULL );
-	res = cut_pixmap( asv, src, trg, x, y, src_w, src_h, width, height, gc, tint );
-	if( asv )
-		destroy_asvisual( asv, False );
+	res = cut_pixmap( get_default_asvisual(), src, trg, x, y, src_w, src_h, width, height, gc, tint );
 #endif		
 	return res ;	
 }
@@ -744,7 +677,7 @@ Pixmap
 cut_win_pixmap (ASVisual *asv, Window win, Drawable src, int src_w, int src_h, int width,
 	  		    int height, GC gc, ARGB32 tint)
 {
-  unsigned int x = 0, y = 0;
+  int x = 0, y = 0;
 
   if (!GetWinPosition (win, &x, &y))
 	return None;
@@ -756,7 +689,7 @@ Pixmap
 CutWinPixmap (Window win, Drawable src, int src_w, int src_h, int width,
 	      int height, GC gc, ShadingInfo * shading)
 {
-  unsigned int x = 0, y = 0;
+  int x = 0, y = 0;
 
   if (!GetWinPosition (win, &x, &y))
 	return None;
